import { ArrayWrapper } from './array';
import { MapWrapper } from './map';
import { ObjectWrapper } from './object';
import { SetWrapper } from './set';
import { Wrapper } from './wrapper';
export declare type Lifted<T> = undefined extends T ? never : null extends T ? never : T extends Wrapper<infer W> ? LiftedValue<W> : LiftedValue<T>;
declare type AtomicObject = Function | Promise<any> | Date | RegExp | Boolean | Number | String;
declare type LiftedValue<T> = null extends T ? never : undefined extends T ? never : T extends AtomicObject ? T : T extends ReadonlyArray<any> ? ArrayWrapper<T> : T extends ReadonlyMap<infer K, infer V> ? MapWrapper<K, V> : T extends ReadonlySet<infer E> ? SetWrapper<E> : T extends object ? ObjectWrapper<T> : never;
interface Lift {
    <T>(obj: ArrayWrapper<ReadonlyArray<T>>): ArrayWrapper<ReadonlyArray<T>>;
    <T extends object>(obj: ObjectWrapper<T>): ObjectWrapper<T>;
    <K, V>(obj: MapWrapper<K, V>): MapWrapper<K, V>;
    <T>(obj: SetWrapper<T>): SetWrapper<T>;
    /** lift won't wrap primitives and some other non container-like objects */
    <T extends AtomicObject>(obj: T): T;
    /** Wraps an Array to provide a richer API. Unwrap with .value() **/
    <T>(obj: T[]): ArrayWrapper<T[]>;
    <T>(obj: ReadonlyArray<T>): ArrayWrapper<ReadonlyArray<T>>;
    /** Wraps a Set to provide a richer API. Unwrap with .value() **/
    <T>(obj: ReadonlySet<T>): SetWrapper<T>;
    /** Wraps a Map to provide a richer API. Unwrap with .value() **/
    <K, V>(obj: ReadonlyMap<K, V>): MapWrapper<K, V>;
    /** Wraps an Object to provide a richer API. Unwrap with .value() **/
    <T extends object>(obj: T): ObjectWrapper<T>;
}
export declare const lift: Lift;
export declare type Pipe = typeof pipe;
export declare function pipe<T, R>(this: Wrapper<T>, func: (object: T) => R): Lifted<R>;
export {};
